"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

ImageView subclass:#ImageEditView
	instanceVariableNames:'magnification colors magnifiedImage colorPanel pathName'
	classVariableNames:''
	poolDictionaries:''
	category:'Views-Misc'
!

!ImageEditView class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    This View will eventually be able to edit bitmap images.
    For now, it is not.

    [author:]
        Claus Gittinger

    [see also:]
        Image Form

    [start with:]
        ImageEditView openOn:'bitmaps/gifImages/garfield.gif'
        ImageEditView openOnImage:(Image fromFile:'bitmaps/gifImages/garfield.gif')
"
! !

!ImageEditView class methodsFor:'startup'!

openOn:aFileName
    "startup an image editor on an image read from a file"

    |editView|

    editView := super openOn:aFileName.
    editView pathName:aFileName asFilename pathName.

    ^ editView

    "
     ImageEditView openOn:'bitmaps/gifImages/garfield.gif'
     ImageEditView openOn:'bitmaps/xpmBitmaps/BOOK.xpm'
    "

    "Modified: 10.4.1997 / 17:31:17 / cg"
! !

!ImageEditView methodsFor:'accessing'!

image:anImage
    |oldMag|

    oldMag := magnification.
    magnification := 1@1.
    magnifiedImage := nil.
    super image:anImage.
    self magnification:oldMag

    "Modified: 20.2.1997 / 18:29:13 / cg"
!

magnification:aMagnificationPoint
    |nPixel savedImage|

    magnification ~= aMagnificationPoint ifTrue:[
        "show wait cursor; although magnification is fast, dithering may take a while"
        self cursor:Cursor wait.

        (device visualType == #PseudoColor) ifTrue:[
            "release kept colors"
            colors := IdentitySet new.
        ].

        magnification := aMagnificationPoint asPoint.

        "use a magnified image, if its size wont be too big"
        magnifiedImage := nil.

        "avoid slow scroll"
        savedImage := image.
        image := nil.
        self scrollToTopLeft.
        image := savedImage.

        magnification ~= (1@1) ifTrue:[
            nPixel := image width * image height * magnification x * magnification y.
            nPixel <= (2000 * 2000) ifTrue:[
                Transcript showCR:'magnifying ..'; endEntry.
                magnifiedImage := image magnifiedBy:magnification
            ].
        ].

        self contentsChanged.
        self clear.
        self invalidate.
        self cursor:Cursor normal
    ]

    "Modified: 5.7.1996 / 17:44:29 / stefan"
    "Modified: 16.10.1996 / 01:06:00 / cg"
!

pathName:aPathName
    "set the pathName of the edited image.
     This is used as a default in the save-dialog"

    pathName := aPathName

    "Created: 10.4.1997 / 17:31:54 / cg"
! !

!ImageEditView methodsFor:'drawing'!

redrawX:x y:y width:w height:h
    |area ih iw dotW dotH minX maxX minY maxY color last lastY runW x0 oldClip|

    image isNil ifTrue:[^ self].

"/    area := Rectangle left:x top:y width:w height:h.      
"/    oldClip := self clippingRectangleOrNil.
"/    self clippingRectangle:area.

    (magnification = (1@1)) ifTrue:[
        super redrawX:x y:y width:w height:h.
"/        self clippingRectangle:oldClip.
        ^ self
    ].
        
"/    self clear.

    magnifiedImage notNil ifTrue:[
        magnifiedImage := magnifiedImage on:device.

        magnifiedImage depth == 1 ifTrue:[
            self paint:(magnifiedImage colorFromValue:1)
                    on:(magnifiedImage colorFromValue:0).
        ] ifFalse:[
            self paint:Black on:White.
        ].

        self function:#copy.

        (image depth ~~ 1
        and:[image mask notNil]) ifTrue:[
            self clearRectangleX:x y:y width:w height:h.
            self displayForm:magnifiedImage x:margin y:margin 
        ] ifFalse:[
            self displayOpaqueForm:magnifiedImage x:margin y:margin 
        ].

"/        self clippingRectangle:oldClip.
        ^ self
    ].

"/    self clear.
    ih := image height.
    iw := image width.
    dotW := magnification x.
    dotH := magnification y.

    minX := x // dotW.
    minY := y // dotW.
    maxX := (x + w) // dotW + 1.
    maxX > iw ifTrue:[
        maxX := iw
    ].
    maxY := (y + h) // dotH + 1.
    maxY > ih ifTrue:[
        maxY := ih
    ].

    lastY := -1.

    x0 := minX.
    runW := 0.

    image colorsFromX:minX y:minY toX:maxX-1 y:maxY-1 do:[:x :y :color |
        y ~~ lastY ifTrue:[
            runW ~~ 0 ifTrue:[
                self fillRectangleX:(x0 * dotW + margin)
                                  y:(lastY * dotH + margin)
                              width:runW height:dotH.
                runW := 0.
            ].
            x0 := x.
            lastY := y.
        ].

        color ~~ last ifTrue:[
            runW ~~ 0 ifTrue:[
                self fillRectangleX:(x0 * dotW + margin)
                                  y:(y * dotH + margin)
                              width:runW height:dotH.
                runW := 0.
            ].

            "/ keep colors - otherwise they might get collected & changed
            colors notNil ifTrue:[colors add:color].
            self paint:color.
            last := color.
            runW := 0.
            x0 := x.
        ].
        runW := runW + dotW
    ].

    runW ~~ 0 ifTrue:[
        self fillRectangleX:(x0 * dotW + margin)
                          y:(lastY * dotH + margin)
                      width:runW height:dotH.
        runW := 0.
    ].

"/    self clippingRectangle:oldClip

    "Modified: 20.2.1997 / 18:27:10 / cg"
! !

!ImageEditView methodsFor:'event handling'!

buttonMotion:state x:x y:y
    self showColorAtX:x y:y.
!

buttonMultiPress:button x:x y:y
    button == 1 ifTrue:[
        |clr|
        clr := self getColorAtX:x y:y.
        clr inspect.
    ].
    super buttonMultiPress:button x:x y:y

    "Created: 8.5.1996 / 00:18:06 / stefan"
!

buttonPress:button x:x y:y
    button == 1 ifTrue:[
	self showColorAtX:x y:y.
	^ self
    ].
    super buttonPress:button x:x y:y
!

getColorAtX:x y:y
    |pi|

    pi := (x @ y) - margin.
    pi := pi // magnification.
    ((0@0 corner:image extent) containsPoint:pi) ifTrue:[
        ^ image at:pi.
    ].
    ^ nil

    "Created: 8.5.1996 / 00:15:55 / stefan"
!

showColorAtX:x y:y
    |clr|

    clr := self getColorAtX:x y:y.
    clr notNil ifTrue:[
        colorPanel notNil ifTrue:[
            colorPanel setColor:clr.
        ] ifFalse:[
            Transcript showCR:clr displayString
        ]
    ]

    "Modified: 8.5.1996 / 00:20:45 / stefan"
! !

!ImageEditView methodsFor:'image processing'!

blurr
    self performImageOperation:#blurr withArguments:nil
!

flipHorizontal
    self performImageOperation:#flipHorizontal withArguments:nil
!

flipVertical
    self performImageOperation:#flipVertical withArguments:nil
!

performImageOperation:operation withArguments:args
    |oldMag|

    windowGroup withCursor:Cursor wait do:[
        oldMag := magnification.
        magnifiedImage := nil.
        magnification := 1@1.
        image := image perform:operation withArguments:args.
        (oldMag isNil or:[oldMag = magnification]) ifTrue:[
            self clear.
            self invalidate
        ] ifFalse:[
            self magnification:oldMag.
        ]
    ]

    "Modified: 17.3.1997 / 11:58:33 / cg"
!

rotateCCW
    self performImageOperation:#rotated: withArguments:#(270)

    "Created: 17.3.1997 / 11:56:42 / cg"
!

rotateCW
    self performImageOperation:#rotated: withArguments:#(90)

    "Created: 17.3.1997 / 11:56:36 / cg"
! !

!ImageEditView methodsFor:'initialization'!

initialize
    super initialize.
    magnification := 1@1.
    colors := nil.

    self menuHolder:self; menuPerformer:self; menuMessage:#imageMenu.

    "Modified: 20.2.1997 / 18:33:51 / cg"
! !

!ImageEditView methodsFor:'menu'!

imageMenu
    |m labels selectors|

    image mask notNil ifTrue:[
        labels := #(
                        'save as ...'
                        'save mask as ...'
                       ).
        selectors := #(
                        saveAs
                        saveMaskAs
                      ).
        magnifiedImage notNil ifTrue:[
            labels := labels , #(
                        '-'
                        'save magnified as ...'
                        'save magnified mask as ...'
                       ).

            selectors := selectors , #(
                        nil
                        saveMagnifiedAs
                        saveMagnifiedMaskAs
                      ).
        ]
    ] ifFalse:[
        labels := #(
                        'save as ...'
                       ).
        selectors := #(
                        saveAs
                      ).
        magnifiedImage notNil ifTrue:[
            labels := labels , #(
                        '-'
                        'save magnified as ...'
                       ).

            selectors := selectors , #(
                        nil
                        saveMagnifiedAs
                      ).
        ]
    ].

    labels := labels , #(
                            '-'
                            'magnification'
"/                            'colors'
                            'effects'
                        ).
    selectors := selectors , #(
                        nil
                        changeMagnification
"/                        showColors
                        effects
                        ).

    m := PopUpMenu
               labels:(resources array:labels)
            selectors:selectors
             receiver:self
                  for:self.

    m subMenuAt:#effects put:(
        PopUpMenu labels:(resources array:#(
                            'flip - vertical'
                            'flip - horizontal'
                            '-'
                            'rotate - clockwise'
                            'rotate - counter clockwise'
"
                            '-'
                            'blurr'
"
                           ))
               selectors:#(
                            flipVertical
                            flipHorizontal
                            nil
                            rotateCW
                            rotateCCW
"
                            nil
                            blurr
"
                           )
                receiver:self
                     for:self

    ).

    ^ m

    "
     ImageEditView openOn:'bitmaps/SBrowser.xbm'
     ImageEditView openOn:'bitmaps/garfield.gif'
    "

    "Created: 20.2.1997 / 18:47:17 / cg"
    "Modified: 17.3.1997 / 11:55:43 / cg"
! !

!ImageEditView methodsFor:'menu actions'!

changeMagnification
    |b|

    b := EnterBox new.
    b title:'magnification (magX @ magY)'.
    b okText:'apply'.
    b abortText:'abort'.
    b action:[:string | self magnification:(Object readFromString:string)].
    b initialText:(magnification printString).
    b showAtPointer
!

doSaveImageAs:anImage title:aTitle
    "save contents into a file 
     - ask user for filename using a fileSelectionBox."

    |fileName defaultName rdr i txt suffix|

    defaultName := pathName ? ''.

    pathName isNil ifTrue:[
        suffix := 'tiff'
    ] ifFalse:[
        suffix := pathName asFilename suffix.
        "/ a supported suffix ?
        ((rdr := Image imageReaderClassForSuffix:suffix) isNil 
        or:[(rdr canRepresent:anImage) not]) ifTrue:[
            suffix := 'tiff'.
            defaultName := (pathName asFilename withSuffix:suffix) pathName
        ].
    ].

    fileName := Dialog
                    requestFileName:(resources string:aTitle)
                    default:defaultName
                    ok:(resources string:'save')
                    abort:(resources string:'abort')
                    pattern:('*.' , suffix).

    fileName notNil ifTrue:[
        anImage saveOn:fileName.
        pathName := fileName.
    ].

    "Created: 20.2.1997 / 18:52:08 / cg"
    "Modified: 10.4.1997 / 17:49:58 / cg"
!

saveAs
    "save contents into a file 
     - ask user for filename using a fileSelectionBox."

    self doSaveImageAs:image title:'save image in:'

    "Modified: 20.2.1997 / 18:52:37 / cg"
!

saveMagnifiedAs
    "save contents into a file 
     - ask user for filename using a fileSelectionBox."

    self doSaveImageAs:magnifiedImage title:'save magnified image in:'

    "Created: 20.2.1997 / 18:52:53 / cg"
!

saveMagnifiedMaskAs
    "save contents into a file 
     - ask user for filename using a fileSelectionBox."

    self doSaveImageAs:(magnifiedImage mask) title:'save magnified mask in:'

    "Created: 20.2.1997 / 18:53:31 / cg"
    "Modified: 20.2.1997 / 18:56:48 / cg"
!

saveMaskAs
    "save contents into a file 
     - ask user for filename using a fileSelectionBox."

    self doSaveImageAs:(image mask) title:'save mask in:'

    "Created: 20.2.1997 / 18:51:06 / cg"
    "Modified: 20.2.1997 / 18:56:54 / cg"
!

showColors
    colorPanel isNil ifTrue:[
        colorPanel := ColorPanel new
    ].
    colorPanel shown ifFalse:[
        colorPanel open
    ]

    "Modified: 10.4.1997 / 17:22:35 / cg"
! !

!ImageEditView methodsFor:'queries'!

heightOfContents
    "return the images height"

    image isNil ifTrue:[^ 0].
    ^ (image height * magnification y) rounded
!

widthOfContents
    "return the images width"

    image isNil ifTrue:[^ 0].
    ^ (image width * magnification x) rounded
! !

!ImageEditView methodsFor:'release'!

destroy
    colors := nil.
    magnifiedImage := nil.
    colorPanel notNil ifTrue:[
	colorPanel destroy.
	colorPanel := nil.
    ].
    super destroy.
! !

!ImageEditView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libwidg2/ImageEditView.st,v 1.35 1997-04-10 15:51:10 cg Exp $'
! !
